
using AspNetCoreApi.DbContexts;
using AspNetCoreApi.Middlewares;
using AspNetCoreApi.Models;
using AspNetCoreApi.Services;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using Microsoft.IdentityModel.Tokens;
using System.Text;

namespace AspNetCoreApi
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);

            // Add EF context
            var connString = "server=peacecloud.synology.me;port=23306;database=Study;uid=study;password=Study1234!";
            var dbServerVersion = new MySqlServerVersion(new Version(10, 3, 32));


            builder.Services.AddDbContext<AppDbContext>(
                options => options
                    .UseMySql(connString, dbServerVersion));

            // Add Dependancy injection
            builder.Services.AddScoped<ProductService>();

            // Add Identity
            builder.Services.AddScoped<ApplicationUserService>();
            builder.Services.AddIdentity<ApplicationUser, IdentityRole>()
                .AddEntityFrameworkStores<AppDbContext>()
                .AddDefaultTokenProviders();

            // JWT Authorization
            string jwtKey = "ThisIsMyMyJWTKey1234!ThisIsMyMyJWTKey1234!";
            builder.Services.AddAuthentication(options =>
            {
                options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            })
            .AddJwtBearer(options =>
            {
                options.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuer = true,
                    ValidateAudience = true,
                    ValidateLifetime = true,
                    ValidateIssuerSigningKey = true,
                    ValidIssuer = "MyIssuer",
                    ValidAudience = "MyAudience",
                    IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(jwtKey))
                };
            });

            builder.Services.AddScoped<JWTAuthenticationService>();

            // Add services to the container.

            builder.Services.AddControllers();
            // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
            builder.Services.AddEndpointsApiExplorer();
            builder.Services.AddSwaggerGen();

            var app = builder.Build();

            // Configure the HTTP request pipeline.
            if (app.Environment.IsDevelopment())
            {
                app.UseSwagger();
                app.UseSwaggerUI();
            }


            // Add Middlewares
            app.UseMiddleware<CustomMiddleware1>();
            app.UseMiddleware<CustomMiddleware2>();

            app.UseHttpsRedirection();

            app.UseAuthentication();
            app.UseAuthorization();


            //// Add Routing
            //app.UseRouting();
            //app.UseEndpoints(endpoints =>
            //{
            //    endpoints.MapGet("/routingtest", async context =>
            //    {
            //        await context.Response.WriteAsync("Hello, World!");
            //    });

            //    endpoints.MapGet("/routingtest/hello/{name}", async context =>
            //    {
            //        var name = context.Request.RouteValues["name"];
            //        await context.Response.WriteAsync($"Hello, {name}!");
            //    });
            //});

            app.MapControllers();

            app.Run();
        }
    }
}
